use warnings;
use strict;
use Test::More tests => 22;
use Test::Exception;

use Time::HiRes qw( sleep );
use POWER::Event::Timer qw(:ALL);
use POWER::Utils;

throws_ok { set_timer()                             } qr/usage: set_timer/;
throws_ok { set_timer(  cb=>sub{}, for=>1     )     } qr/usage: set_timer/;
throws_ok { set_timer({            for=>1    })     } qr/{cb} required/;
throws_ok { set_timer({ cb=>sub{}            })     } qr/{for} or {every} required/;
throws_ok { set_timer({ cb=>sub{}, for=>1, every=>1 
                                             })     } qr/{for} or {every} required/;
throws_ok { set_timer({ cb=>sub{}, for=>undef   })  } qr/{for} undefined/;
throws_ok { set_timer({ cb=>sub{}, every=>undef })  } qr/{every} undefined/;
throws_ok { set_timer({ cb=>sub{}, for=>0    })     } qr/wrong {for}/;
throws_ok { set_timer({ cb=>sub{}, for=>-1   })     } qr/wrong {for}/;
throws_ok { set_timer({ cb=>sub{}, every=>0  })     } qr/wrong {every}/;
throws_ok { set_timer({ cb=>sub{}, every=>-1 })     } qr/wrong {every}/;
throws_ok { set_timer({ cb=>sub{}, for=>1, x=>1 })  } qr/unknown param {x}/;

my ($t1, $t2);
lives_ok  { $t1 = set_timer({ cb=>sub{}, for=>1   }) } q{set one-shot timer};
lives_ok  { $t2 = set_timer({ cb=>sub{}, every=>1 }) } q{set repeated timer};

throws_ok { reset_timer()                           } qr/usage: reset_timer/;
throws_ok { reset_timer(sub{})                      } qr/usage: reset_timer/;

lives_ok  { reset_timer($t1)                         } q{can reset timer};

throws_ok { stop_timer()                            } qr/usage: stop_timer/;
throws_ok { stop_timer(sub{})                       } qr/usage: stop_timer/;

lives_ok  { stop_timer($t1)                          } q{can stop timer};

throws_ok { stop_timer($t1)                         } qr/no such timer/;

set_timer({ for => 0.01, cb => 'main', method => 'no_such_method' });
sleep 0.01;
throws_ok { run_callback(@$_) for get_expired_timers() }
    qr/Can't locate object method "no_such_method" via package "main"/;

